#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

# This file is part of Oidua.
#
# Oidua is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Copyright 2003	Sylvester Johansson 	(sylvestor@telia.com)
#			Mattias Pivrinta	(mpa99001@student.mdh.se)


"""Usage:\toidua.py [options] <basedir> ...

Options:
     -C n	Set output width to n characters (Default: 80)
     -D		Display datestamp header
     -e dir	Exclude dir from search
     -h		Display this message
     -H		HTML output
     -i		Case-insensitive directory sorting
     -p[=3mof]	Guess preset. Specify preset guessing on specific
        	formats. (mp[3], [o]gg, [m]pc, [f]lac) (Default: all)
     -S		Display statistics results
     -t		Display elapsed time footer
     -V		Display version
     --text=x	Set HTML text color to x.
     --bg=x	Set HTML background color to x.
"""

__version__ = "0.14"

import sys,string,re,os,time,audiotype


#initial values

class Data:
	def __init__(self):
		self.Width = 		80
		self.Preset = 		0
		self.Base = 		0
		self.DispVersion = 	0
		self.DispTime = 	0
		self.DispHelp = 	0
		self.DispDate = 	0
		self.DispResult = 	0
		self.OutputFormat =	0
		self.Start =		0
		self.CaseInsensitiveSort = 0
		self.TotalSize = 	0.0
		self.TotalOGG = 	0.0
		self.TotalMP3 = 	0.0
		self.TotalMPC = 	0.0
		self.TotalFLAC = 	0.0
		self.TimeTotal =	0.0
		self.TimeOGG =		0.0
		self.TimeMP3 =		0.0
		self.TimeMPC =		0.0
		self.TimeFLAC =		0.0
		self.BGColor = 		"white"
		self.TextColor =	"black"
		
		self.Folders = 		[]
		self.ExcludePaths = 	[]
		
def main():
	#sys.argv processing
	for x in range(len(sys.argv)):
		if sys.argv[x][0] == "-":
			for y in sys.argv[x]:
				if y == "t":
					Settings.DispTime = 1
				if y == "V":
					Settings.DispVersion = 1
				if y == "D":
					Settings.DispDate = 1
				if y == "h":
					Settings.DispHelp = 1
				if y == "H":
					Settings.OutputFormat = 1
				if y == "S":
					Settings.DispResult = 1
				if y == "p":
					Settings.Preset = 15
				if y == "i":
					Settings.CaseInsensitiveSort = 1
		if sys.argv[x] == "-C":
			try:
				Settings.Width == string.atoi(sys.argv[x+1])
			except:
				print sys.argv[x+1],"is not a integer"
		if sys.argv[x] == "-e":
			if os.path.isdir(sys.argv[x+1]):
				Settings.ExcludePaths.append(sys.argv[x+1])
		if sys.argv[x][:2] == "-p":
			if len(sys.argv[x]) == 2:
				Settings.Preset = 15
			elif sys.argv[x][2] == "=":
				Settings.Preset = presetbyte(sys.argv[x][3:])
		if os.path.isdir(sys.argv[x]):
			if sys.argv[x-1] != "-e":
				Settings.Folders.append(sys.argv[x])
		if sys.argv[x][:6] == "--text":
			Settings.TextColor = sys.argv[x][7:]
		if sys.argv[x][:4] == "--bg":
			Settings.BGColor = sys.argv[x][5:]
	if Settings.DispHelp:
		print __doc__
		return 0
	if Settings.DispTime == 1:
		Settings.Start = time.clock()
	if Settings.OutputFormat == 1:
		htmlheader()		
	if Settings.DispDate == 1:
		headers("date")
	if Settings.Folders:
		headers("header")
		for x in Settings.Folders:
			headers("sep")
			x = os.path.abspath(x)
			Settings.Base = string.count(x[:-1],os.sep)
			grab(x)
			smash(x)
	elif not Settings.DispHelp and not Settings.DispVersion:
		print "No folders to process. Type \'oidua.py -h\' for more information."
		return 0
	Settings.ElapsedTime = time.clock() - Settings.Start
	if Settings.DispTime:
		print ''
		print "Generation time:     %8.2f s" % Settings.ElapsedTime
	if Settings.DispResult:
		print ''
		print '+-----------------------+-----------+'
		print '| Format    Amount (Mb) | Ratio (%) |'
		print '+-----------------------+-----------+'
		if Settings.TotalOGG:
			print "| Ogg      %12.2f | %9.2f |" % (Settings.TotalOGG / (1024*1024), Settings.TotalOGG * 100 / Settings.TotalSize)
		if Settings.TotalMP3:
			print "| MP3      %12.2f | %9.2f |" % (Settings.TotalMP3 / (1024*1024), Settings.TotalMP3 * 100 / Settings.TotalSize)
		if Settings.TotalMPC:
			print "| MPC      %12.2f | %9.2f |" % (Settings.TotalMPC / (1024*1024), Settings.TotalMPC * 100 / Settings.TotalSize)
		if Settings.TotalFLAC:
			print "| FLAC     %12.2f | %9.2f |" % (Settings.TotalFLAC / (1024*1024), Settings.TotalFLAC * 100 / Settings.TotalSize)
		print '+-----------------------+-----------+'
		print "| Total %10.2f Mb   |" % (Settings.TotalSize / (1024 * 1024))
		if Settings.ElapsedTime and Settings.TotalSize:
			print "| Speed %10.2f Mb/s |" % (Settings.TotalSize / (Settings.ElapsedTime * 1024 * 1024))
		else:
			print "| Speed        N/A      |"
		print '+-----------------------+'
	if Settings.DispVersion:
		print "\n%-20s %s\n%-20s %s" % ("oidua version: ",__version__,"audiotype version: ",audiotype.__version__)
	if Settings.OutputFormat == 1:
		htmlfooter()
		
def htmlheader():
	print "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">"
	print "<html>"
	print "<head>"
	print "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">"
	print "<title>Muzak List</title>"
	print "<!-- Generated by oidua",__version__,"-->"
	print "<!-- http://legolas.mdh.se/~dal99mpa/oidua.html -->"
	print "<style type=\"text/css\">"
	print "  body { color:",Settings.TextColor,"; background:",Settings.BGColor,"; }"
	print "</style>"
	print "</head>"
	print "<body>"
	print "<pre>"

def htmlfooter():
	print "</pre>"
	#print "<p><a href=\"http://validator.w3.org/check/referer\">"
	#print "<img src=\"http://www.w3.org/Icons/valid-html401\" alt=\"Valid HTML 4.01!\" height=\"31\" width=\"88\"></a></p>"
    	print "</body></html>"

def presetbyte(formatstring):
	byte = 0
	for x in formatstring:
		if string.lower(x) == "o":
			byte = byte | 1
		if string.lower(x) == "3":
			byte = byte | 2
		if string.lower(x) == "m":
			byte = byte | 4
		if string.lower(x) == "f":
			byte = byte | 8
	return byte

def sorted(fulldir, pattern):
	f = re.compile(pattern, re.IGNORECASE)
	temp = []
	for x in fulldir:
		if f.search(x):
			temp.append(x)
	return temp

def average(listofbr):
	temp = 0
	for x in listofbr:
		temp += x
	return temp/len(listofbr)

def vbrchecker(vbrlist):
	if len(vbrlist) == 1:
		if (vbrlist[0] == 0):
			return "C"
		if (vbrlist[0] == 1):
			return "V"
	else:
		return "~"

def headers(token):
	if token == "header": #top header
		outputstring = "%-" + str(Settings.Width-29) + "s %6s %s %-4s %s %s"
		print outputstring%("Album/Artist","Mb",'|',"Type",'|',"Encoding")
	if token == "sep": #separator
		print ("="*(Settings.Width-21)+'+'+("="*6)+'+'+("="*11))
	if token == "date": #date
		outputstring = "%"+str(Settings.Width-1)+"s" 
		print outputstring%(time.strftime("%a %b %d %H:%M:%S %Y",time.localtime()))

def outputter(trailcount,folder = "", albumsize = 0.0, bitrates =[], presets = [], vbrtuple = [], filetypes = []):
	outputfolder = ("   " * trailcount) + folder
	if bitrates:
		if (Settings.Preset & (1 << filetypes[0])) and (len(presets) == 1):
			outputstring = "%-" + str(Settings.Width - 29) + "s %6.1f %s %-4s %s %s"
			print outputstring%(outputfolder[:Settings.Width-30], albumsize/(1024*1024), '|', fileformat(filetypes), '|', presets[0])
		else:
			outputstring = "%-" + str(Settings.Width - 29) + "s %6.1f %s %-4s %s %4d %s"
			print outputstring%(outputfolder[:Settings.Width-30], albumsize/(1024*1024), '|', fileformat(filetypes),'|', average(bitrates), vbrchecker(vbrtuple))
	else:
		outputstring = "%-" + str(Settings.Width - 22) + "s %s %4s %s"
		print outputstring%(outputfolder[:Settings.Width-29],'|','','|')

def fileformat(formats):
	back = []
	if len(formats)<2:
		for x in formats:
			if x == 0:
				try:
					back.index("Ogg")
				except:
					back.append("Ogg")
			if x == 1:
				try:
					back.index("MP3")
				except:
					back.append("MP3")
			if x == 2:
				try:
					back.index("MPC")
				except:
					back.append("MPC")
			if x == 3:
				try:
					back.index("FLAC")
				except:
					back.append("FLAC")
	else:
		back.append("Mix")
	return string.join(back,",")

def grab(folder):
	temp = os.listdir(folder)
	files = []
	for x in temp:
		if os.path.isfile(os.path.join(folder,x)):
			files.append(x)
	bitrates = []
	albumsize = 0.0
	format = []
	vbrtuple = []
	presets = []

	for x in sorted(files, "(ogg|mpc|mp3|flac)\Z"):
		if(string.lower(x[-3:]) == "mp3"):
			file = audiotype.MP3(os.path.join(folder,x))
			Settings.TotalMP3 += file.size
		if(string.lower(x[-3:]) == "mpc"):
			file = audiotype.MPC(os.path.join(folder,x))
			Settings.TotalMPC += file.size
		if(string.lower(x[-3:]) == "ogg"):
			file = audiotype.OGG(os.path.join(folder,x))
			Settings.TotalOGG += file.size
		if(string.lower(x[-4:]) == "flac"):
			file = audiotype.FLAC(os.path.join(folder,x))
			Settings.TotalFLAC += file.size

		albumsize += file.size
		bitrates.append(file.bitrate)

		try:
			format.index(file.format)
		except:
			format.append(file.format)
		try:
			vbrtuple.index(file.vbr)
		except:
			vbrtuple.append(file.vbr)
		if file.profile():
			try:
				presets.index(file.profile())
			except:
				presets.append(file.profile())

	outputter(string.count(folder,os.sep)-Settings.Base, os.path.split(folder)[-1], albumsize, bitrates, presets, vbrtuple, format)	
	Settings.TotalSize += albumsize

def smash(found):
	test = os.listdir(found)
	if Settings.CaseInsensitiveSort:
		test.sort(lambda x,y: cmp(string.lower(x),string.lower(y)))
	else:
		test.sort()
	for x in test:
		next = os.path.join(found,x)
		try:
			if Settings.ExcludePaths:
				try:
					Settings.ExcludePaths.index(next)
				except:
					grab(next)
					smash(next)
			else:
				grab(next)
				smash(next)
		except:
			pass


if __name__ == "__main__":
	Settings = Data()
	main()
